/*

  The contents of this file are subject to the Mozilla Public License Version
  1.1 (the "License"); you may not use this file except in compliance with
  the License. You may obtain a copy of the License at

           http://www.mozilla.org/MPL/

  Software distributed under the License is distributed on an "AS IS" basis,
  WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
  for the specific language governing rights and limitations under the License.

  The Original Code is ActionScriptFacile Framework.

  The Initial Developer of the Original Code is
  Matthieu  - http://www.actionscript-facile.com
  Portions created by the Initial Developer are Copyright (C) 2010
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

*/

package
{
	import flash.events.MouseEvent;

	import com.actionscriptfacile.skin.ISkin;

	import flash.utils.getDefinitionByName;

	import org.osflash.thunderbolt.Logger;

	import com.as3facileexemple.skin.classic.DefaultButtonSkin;
	import com.as3facileexemple.skin.classic.DefaultButtonSkin2;
	
	import com.actionscriptfacile.ui.button.Button;
	import com.actionscriptfacile.ui.UIComponent;
	
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.text.TextField;
	import flash.utils.Dictionary;
	

	/**
	 * Exemple de Mini Flex avec les composants graphiques ActionScript-Facile.
	 * 
	 * @author Matthieu
	 */
	 public class MiniFlexExample extends Sprite
	{	
		private var m_correspTable:Dictionary;// création d'une table de correspondance.
		
		/*
		 * Permet de forcer l'inclusion des classes de Skin Button dans le swf
		 */
        public static var linkages:Array = [DefaultButtonSkin, DefaultButtonSkin2, MouseEvent.CLICK];
        
        
		public function MiniFlexExample() 
		{
			Logger.debug ("info a="+Math.random()*28); // debug de l'application avec l'extension firefox Firebug

			// chargement le fichier XML
			var loader:URLLoader = new URLLoader();
			loader.addEventListener(Event.COMPLETE, completeHandler );
			loader.addEventListener(IOErrorEvent.IO_ERROR, errorHandler );
			
			// chargement du fichier xml et gestion du cache du navigateur
			var file:URLRequest =  new URLRequest('interface.xml?a='+Math.random()*28);
			loader.load(file );
		}
		
		private function errorHandler(e:IOErrorEvent):void 
		{
			// gestion des erreurs
			var loader:URLLoader = e.target as URLLoader;
			loader.removeEventListener(Event.COMPLETE, completeHandler );
			loader.removeEventListener(IOErrorEvent.IO_ERROR, errorHandler );
			
			var text:TextField = addChild( new TextField() ) as TextField;
			text.width = stage.stageWidth;
			text.autoSize = 'center';
			text.text = "Erreur : "+e.text;
		}
		
		private function completeHandler(e:Event):void 
		{
			// récupération des données puis transmission de celles-ci à la méthode parseInterface
			var loader:URLLoader = e.target as URLLoader;
			loader.removeEventListener(Event.COMPLETE, completeHandler );
			loader.removeEventListener(IOErrorEvent.IO_ERROR, errorHandler );
			
			parseInterface( new XML( loader.data ) );
		}
		
		
		/**
		 * Analyse du fichier xml et construction de l'interface.
		 * 
		 */
		private function parseInterface( p_interface:XML ):void
		{
			// on référence le composant button dans la table de correspondance
			m_correspTable = new Dictionary();
			m_correspTable['button'] = buildButton;// appel automatique de la méthode de construction d'un Button
			
			var component:UIComponent;
			var func:Function;
		
			XML.ignoreComments = true;
			XML.ignoreWhitespace = true;
			XML.ignoreProcessingInstructions = true;
			
			for each ( var xml:XML in p_interface.children() )
			{
				// on cherche la fonction de parsing du composant
				func = m_correspTable[ xml.name() ] as Function;
				
				// si on ne la trouve pas on continue
				if ( func == null )
					continue;
				
				// sinon on  l'éxécute
				component = func( xml ) as UIComponent;// création du composant
				
				// on écoute les changements qui interviennent sur le composant
				component.addEventListener(MouseEvent.CLICK, updateLabel );
			
				// et on ajoute le composant à la scène
				addChild ( component );
			}
		}
		
		
		/**
		 * Modifie la label du Button
		 * 
		 */
		private function updateLabel(e:MouseEvent): void 
		{
			var oButton:Button = e.target as Button;
			oButton.label = "N° " + Math.ceil(Math.random()*28);
			
			Logger.debug ("updateLabel : "+oButton.label);
		}

		/**
		 * fonction permettant de construire un composant button
		 * 
		 */
		private function buildButton( p_xml:XML ):Button
		{
			
			var prop:XML;
			var l_labelField:XML;
			var l_labelsList:XMLList = p_xml.child('labelField');
					
			var l_component:Button = new Button();
			var parameters:Array = new Array();
			var parameter:XML;
			var method:Function;
			
			// on regarde si l'objet labelField est défini dans le XML
			if ( l_labelsList.length() > 0 )
			{
				// si oui on le récupère
				l_labelField = l_labelsList[0];
			}
						
			// récupère le skin du fichier xml et l'applique au composant
			var definition:Class = getDefinitionByName(p_xml.attribute('skin') ) as Class;
			l_component.applySkin( new definition() as ISkin);
			
			Logger.debug ("p_xml : "+p_xml.attribute('skin'));
			
			l_component.resize( Number ( p_xml.attribute('width') ), Number ( p_xml.attribute('height') ) );
			
			for each( prop in p_xml.attributes() )
			{
				// on a déjà défini le width et le height
				if ( prop.name() == "width" || prop.name() == "height" )
					continue;
					
					
				// on vérifie que la propriété existe avant de lui appliquer la valeur définit dans le XML
				if ( l_component.hasOwnProperty( new String( prop.name() ) ) )
				{
					l_component[ new String( prop.name() ) ] = prop.valueOf();
				}
			}
			
			// si le labelField est défini
			if ( l_labelField != null )
			{
				// on boucle sur ses propriétés
				for each( prop in l_labelField.attributes() )
				{
					// on regarde si la propriété existe et le cas échéant on lui applique la valeur
					if ( l_component.labelField.hasOwnProperty( new String( prop.name() ) ) )
					{
						l_component.labelField[new String( prop.name() )] = prop.valueOf();
					}
				}
				
				for each( prop in l_labelField.children() )
				{
					
					// on regarde si le noeud est un appel de méthode
					if ( prop.name() == "callMethod" )
					{
						if ( l_component.labelField.hasOwnProperty( prop.attribute('name').toString() ) )
						{
							
							// on récupère les paramètres
							parameters = new Array();
							for each( parameter in prop.children() )
							{
								parameters.push( parameter.valueOf() );
							}
							
							method = l_component.labelField[prop.attribute('name').toString()] as Function;

							if ( method != null )
							{
								// et on appelle la méthode
								method.apply( this, parameters );
							}
							
						}
					}
				}
			}			
			return l_component;// renvoie une référence vers le composant créé
		}

	}
}